struct t_vert3
{
    float x,y,z;
	t_vert3& operator == (t_vert3 &v) {x=v.x;y=v.y;z=v.z;return *this;}
	t_vert3& operator = (float f) {x=y=z=f;return *this;}
	t_vert3& operator += (t_vert3 &v) {x+=v.x;y+=v.y;z+=v.z;return *this;}
	t_vert3& operator -= (t_vert3 &v) {x-=v.x;y-=v.y;z-=v.z;return *this;}
	t_vert3& operator *= (t_vert3 &v) {x*=v.x;y*=v.y;z*=v.z;return *this;}
	t_vert3& operator /= (t_vert3 &v) {x/=v.x;y/=v.y;z/=v.z;return *this;}
};

struct t_poly
{
    int verts[3];
	t_vert3 norm;
};

struct t_uvmap
{
	char	*name;
	float	*u,*v;
	UINT	*index;
	t_uvmap *prev,*next;
};

struct t_wmap
{
	char	*name;
	float	*w;
	UINT	*index;
	t_wmap *prev,*next;
};

struct t_mesh
{
    t_vert3*	verts;
	t_vert3*	norms;
    t_poly*		polys;
    int numverts;
    int numpolys;
	t_mesh *next;
	t_mesh *prev;
};

struct t_plist
{
	UINT	*list;
	t_plist	*next;
	t_plist	*prev;
};

struct t_surf
{
	char *name;
	float dr,dg,db,da;
	t_surf	*next;
	t_surf	*prev;
};


struct t_object
{
	t_mesh*		meshes;
	t_uvmap*	uvmap;
	t_wmap*		wmap;
	t_plist*	plist;
	t_surf*		surf;
    int numverts;
    int numpolys;
	int nummeshes;
	int numuvmaps;
	int numwmaps;
	int numplists;
	int numsurfs;

	void Delete()
	{
		int j=0;

		//Delete Mesh Layers
		t_mesh *tm = meshes;
		if (tm)
		for (j=0; j<nummeshes; j++)
		{
			if (tm->norms) delete (tm->norms);
			delete (tm->polys);
			delete (tm->verts);
			if (!tm->next) break;
			tm = tm->next;
			delete (tm->prev);
		}
		delete (tm);

		//Delete UVMAPs
		t_uvmap *tuv = uvmap;
		if (tuv)
		for (j=0; j<numuvmaps; j++)
		{
			delete (tuv->name);
			delete (tuv->u);
			delete (tuv->v);
			delete (tuv->index);
			if (!tuv->next) break;
			tuv = tuv->next;
			delete (tuv->prev);
		}
		delete (tuv);

		//Delete WMAPs
		t_wmap *tw = wmap;
		if (tw)
		for (j=0; j<numwmaps; j++)
		{
			delete (tw->name);
			delete (tw->w);
			delete (tw->index);
			if (!tw->next) break;
			tw = tw->next;
			delete (tw->prev);
		}
		delete (tw);
		//delete (wmap);

		//Delete PLISTs
		t_plist *tpl = plist;
		if (tpl)
		for (j=0; j<numplists; j++)
		{
			delete (tpl->list);
			if (!tpl->next) break;
			tpl = tpl->next;
			delete (tpl->prev);
		}
		delete (tpl);

		//Delete PLISTs
		t_surf *ts = surf;
		if (ts)
		for (j=0; j<numsurfs; j++)
		{
			delete (ts->name);
			if (!ts->next) break;
			ts = ts->next;
			delete (ts->prev);
		}
		delete (ts);

	}
};

//extern int Load3DS(char* filename, t_model *mod);
//extern int LoadLWO2(char* filename, t_model *mod);
extern BOOL LoadLWO2( char* filename, t_object *g_obj );